import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_trace_measure.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';

class URMTraceVelMeasureCal extends URMTraceMeasureCal {
  URMTraceVelMeasureCal(super.ref);

  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    super.handleURMSimpleMeasureParams();
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;

      URMMeasureProcessResult? outresult =
          await urmApplication.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMVelMeasure,
        rOIType: URMROIType.URMTrace,
        srcDPoints: param!.srcDPoints,
      );
      if (outresult == null) {
        return;
      }
      URMVelMeasureResult? result = outresult.velMeasureResult;

      if (result != null) {
        final feature = ref.feature!;
        feature.updateStringValue(
            ItemOutputMeta("Placeholder", "Placeholder", VidUsUnit.None),
            "",
            VidUsUnit.None);
        for (var output in ref.meta.outputs) {
          if (output.name == MeasureTerms.URMVelMax) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.maxVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelMin) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.minVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelMean) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.meanVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelStd) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.varianceVel, output.unit);
          } else if (output.name == MeasureTerms.Area) {
            output.unit = VidUsUnit.cm2;
            feature.updateFloatValue(output, result.roiArea, output.unit);
          }
        }
      } else {
        throw Exception("URM Measure API error");
      }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }
}
