import 'package:fis_measure/interfaces/enums/items.dart';
import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/workspace/point_info.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_vessel_measure.dart';
import 'package:fis_measure/process/primitives/straightline.dart';
import 'package:fis_measure/utils/prompt_box.dart';
import 'package:fis_measure/view/gesture/cross_position_indicator.dart';

class URMVesselMeasure extends StraightLine {
  URMVesselMeasure(super.meta, super.parent);

  static URMVesselMeasure createURMVesselMeasure(ItemMeta meta,
      [IMeasureItem? parent]) {
    URMVesselMeasure measureVessel = URMVesselMeasure(meta, parent);
    measureVessel.calculator = URMVesselMeasureCal(measureVessel);

    return measureVessel;
  }

  @override
  bool onExecuteMouse(PointInfo args) {
    if (state == ItemStates.finished) {
      if (args.pointType == PointInfoType.mouseDown) {
        state = ItemStates.waiting;
      } else {
        return false;
      }
    }

    if (state == ItemStates.waiting) {
      if (args.pointType == PointInfoType.mouseDown) {
        handleMouseDownWhileWaiting(args);
      }
    } else if (state == ItemStates.running) {
      if (args.pointType == PointInfoType.mouseUp) return false;

      feature?.endPoint = args;
      doCalculate();
      if (args.pointType == PointInfoType.mouseDown) {
        handleFinish();

        ///重置十字样式
        changeCrossIndicatorStyle(CrossIndicatorStyle.nomal);
      }
    }
    return true;
  }

  @override
  void handleMouseDownWhileWaiting(PointInfo args) {
    // TODO: 判断是否当前area
    // 转换为Area逻辑位置
    final point = args.toAreaLogicPoint();
    // feature = StraightLineFeature(this, point, point);
    if (args.hostVisualArea != null) {
      handleTissueTM(args.hostVisualArea!.mode.modeType, point);
      feature!.hostVisualArea = args.hostVisualArea;
    }
    state = ItemStates.running;
  }

  bool waitingResult = false;

  void handleFinish() async {
    feature!.isActive = false;
    waitingResult = true;
    await doCalculateAsync();
    doFeatureFinish();
    PromptBox.dismiss();
    waitingResult = false;
  }
}
